@@ -5,7 +5,7 @@ from abc import ABC, abstractmethod
 
 from napoleontoolbox.file_saver import dropbox_file_saver
 
-from napoleontoolbox.forecasting import forecasting_model
+from napoleontoolbox.forecasting import forecasting_utility
 from sklearn.metrics import mean_squared_error
 from sklearn.metrics import accuracy_score
 from sklearn.metrics import confusion_matrix
@@ -22,7 +22,7 @@ import numpy as np
 
 
 class AbstractRunner(ABC):
-    def __init__(self, starting_date = None, running_date = None, drop_token=None, dropbox_backup = True, underlying=None, frequence = None, algo=None, aggregated_pkl_file_suffix='freqly_to_mix.pkl', aggregated_pkl_mapping_file_suffix = 'freqly_to_mix_mapping.pkl', list_pkl_file_suffix = 'my_list.pkl', local_root_directory='../data/', user = 'napoleon'):
+    def __init__(self, starting_date = None, running_date = None, drop_token=None, dropbox_backup = True, underlying=None, frequence = None, algo=None, aggregated_pkl_file_suffix='freqly_to_mix.pkl', aggregated_pkl_mapping_file_suffix = 'freqly_to_mix_mapping.pkl', list_pkl_file_suffix = 'my_list.pkl', local_root_directory='../data/', user = 'napoleon',number_per_year=252):
         super().__init__()
         self.aggregated_pkl_file_suffix=aggregated_pkl_file_suffix
         self.aggregated_pkl_mapping_file_suffix=aggregated_pkl_mapping_file_suffix
@@ -41,6 +41,7 @@ class AbstractRunner(ABC):
         self.running_date = running_date
         self.starting_date = starting_date
         self.starting_iterations_to_pass = 20
+        self.number_per_year = number_per_year
 
 
 
@@ -50,25 +51,22 @@ class AbstractRunner(ABC):
 
 class ForecasterRunner(AbstractRunner):
     def runTrial(self, saver, seed, n, s, s_eval, calibration_step, optimize_threshold, signal_type, idios_string):
+
+
+        continuous_saving_key, two_states_saving_key, three_states_saving_key = saver.create_saving_key(seed, n, s, s_eval, calibration_step, optimize_threshold, signal_type, idios_string)
+
+        check_run_existence, table_number = saver.checkRunExistence(continuous_saving_key)
+        exhaustive_check_run_existence, exhaustive_table_number = saver.exhaustiveCheckRunExistence(continuous_saving_key)
+        assert check_run_existence == exhaustive_check_run_existence
+        if check_run_existence:
+            assert table_number == exhaustive_table_number
+        if check_run_existence:
+            return
+
         if n == 0:
             n = None
         if s_eval == 0:
             s_eval = None
-        idios = json.loads(idios_string)
-        common_params ={
-            'n':n,
-            's':s,
-            's_eval':s_eval,
-            'calibration_step':calibration_step,
-            'optimize_threshold':optimize_threshold,
-            'signal_type' : signal_type
-        }
-        common_params.update(idios)
-        saving_key = json.dumps(common_params, sort_keys=True)
-        saving_key = signal_utility.convert_to_sql_column_format(saving_key)
-        check_run_existence = saver.checkRunExistence(saving_key)
-        if check_run_existence:
-            return
 
         if self.dropbox_backup:
             aggregated_csv_file_name = self.aggregated_pkl_file_name.replace('pkl', 'csv')
@@ -96,10 +94,10 @@ class ForecasterRunner(AbstractRunner):
         print('output shape')
         print(y.shape)
         chosen_method = signal_type
-        model = forecasting_model.ForecasterWrapper(method=chosen_method)
+        model = forecasting_utility.instantiate_model(method=chosen_method)
 
         # Compute rolling weights
-        forecasted_series, features_importances = time_series_forecasting.rolling_forecasting(
+        forecasted_series, features_importances, discrete_two_states_forecasting_series, discrete_three_states_forecasting_series = time_series_forecasting.rolling_forecasting(
             model,
             X,
             y,
@@ -107,20 +105,25 @@ class ForecasterRunner(AbstractRunner):
             s=s,
             s_eval = s_eval,
             calibration_step = calibration_step,
-            optimize_threshold = optimize_threshold,
             method = chosen_method,
             display = True)
 
-        print('forecasting done')
+        print('forecasting backtest done')
         features_importances = features_importances.sum(axis = 0)
         print('features importance')
         print(features_importances.sum())
 
         forecasted_series[np.isnan(forecasted_series)] = 0
         forecasted_series[np.isinf(forecasted_series)] = 0
-        matrix = confusion_matrix(y > 0, forecasted_series > 0)
+        discrete_two_states_forecasting_series[np.isnan(discrete_two_states_forecasting_series)] = 0
+        discrete_two_states_forecasting_series[np.isinf(discrete_two_states_forecasting_series)] = 0
+        discrete_three_states_forecasting_series[np.isnan(discrete_three_states_forecasting_series)] = 0
+        discrete_three_states_forecasting_series[np.isinf(discrete_three_states_forecasting_series)] = 0
+        y[np.isnan(y)] = 0
+        y[np.isinf(y)] = 0
+        matrix = confusion_matrix(y > 0, discrete_two_states_forecasting_series)
         rmse = mean_squared_error(y, forecasted_series)
-        accuracy = accuracy_score(y > 0, forecasted_series > 0)
+        accuracy = accuracy_score(y > 0, discrete_two_states_forecasting_series)
 
 
         print('rmse '  + str(rmse))
@@ -129,26 +132,35 @@ class ForecasterRunner(AbstractRunner):
         print('confusion matrix')
         print(matrix)
 
-        # print(forecasted_series.shape)
-        # if not filter_best_predictors:
-        #     features_importances.to_pickle(local_root_directory + 'importances.pkl')
-        #
-        # perf_df = signal_utility.reconstitute_prediction_perf(y_pred=forecasted_series>0,y_true = y, transaction_cost=False,
-        #                                                     print_turnover=False)
-        #
-        # perf_df = perf_df[perf_df.index >= '2002-02-01']
-        # perf_df.to_pickle(local_root_directory +'perf_df.pkl')
-        # #perf_df = pd.read_pickle(local_root_directory +'perf_df.pkl')
-        # perf_df[['reconstituted_perf','reconstituted_close']].plot()
 
-
-        perf_df = signal_utility.reconstitute_prediction_perf(y_pred=forecasted_series>0, y_true = y, transaction_cost=False, print_turnover=False)
-        sharpe_strat = metrics.sharpe(perf_df['perf_return'].dropna(), period= 252, from_ret=True)
-        sharpe_under = metrics.sharpe(perf_df['close_return'].dropna(), period= 252, from_ret=True)
+        #saving the continuous signal
+        perf_df = signal_utility.reconstitute_prediction_perf(y_pred=forecasted_series, y_true = y, transaction_cost=True, print_turnover=False)
+        sharpe_strat = metrics.sharpe(perf_df['perf_return'].dropna(), period= self.number_per_year, from_ret=True)
+        sharpe_under = metrics.sharpe(perf_df['close_return'].dropna(), period= self.number_per_year, from_ret=True)
         print('underlying sharpe')
         print(sharpe_under)
         print('strat sharpe')
         print(sharpe_strat)
-        saver.saveAll(saving_key, perf_df)
+        saver.saveAll(continuous_saving_key, perf_df)
 
 
+        #saving the 2 states signal
+        perf_df = signal_utility.reconstitute_prediction_perf(y_pred= discrete_two_states_forecasting_series , y_true = y, transaction_cost=True, print_turnover=False)
+        sharpe_strat = metrics.sharpe(perf_df['perf_return'].dropna(), period= self.number_per_year, from_ret=True)
+        sharpe_under = metrics.sharpe(perf_df['close_return'].dropna(), period= self.number_per_year, from_ret=True)
+        print('underlying sharpe')
+        print(sharpe_under)
+        print('strat sharpe')
+        print(sharpe_strat)
+        saver.saveAll(two_states_saving_key, perf_df)
+
+
+        #saving the 3 states signal
+        perf_df = signal_utility.reconstitute_prediction_perf(y_pred=discrete_three_states_forecasting_series, y_true = y, transaction_cost=True, print_turnover=False)
+        sharpe_strat = metrics.sharpe(perf_df['perf_return'].dropna(), period= self.number_per_year, from_ret=True)
+        sharpe_under = metrics.sharpe(perf_df['close_return'].dropna(), period= self.number_per_year, from_ret=True)
+        print('underlying sharpe')
+        print(sharpe_under)
+        print('strat sharpe')
+        print(sharpe_strat)
+        saver.saveAll(three_states_saving_key, perf_df)
\ No newline at end of file
