@@ -12,15 +12,23 @@ from napoleontoolbox.signal import signal_utility
 from napoleontoolbox.parallel_run import signal_result_analyzer
 import json
 
+from pathlib import Path
 
 class AbstractRunner(ABC):
-    def __init__(self, starting_date = None, running_date = None, drop_token=None, dropbox_backup = True, underlying = None, freqly_return_pkl_filename_suffix='freqly_candels.pkl',aggregated_pkl_file_name='freqly_to_mix.pkl', aggregated_pkl_mapping_file_name = 'freqly_to_mix_mapping.pkl', list_pkl_file_name = 'my_list.pkl', local_root_directory='../data/', user = 'napoleon', add_turn_over = False, skipping_size=200, recompute_all = True, db_path_suffix=None):
+    def __init__(self, starting_date = None, running_date = None, drop_token=None, dropbox_backup = True, underlying = None, frequence='daily', algo='' ,freqly_return_pkl_filename_suffix='freqly_candels.pkl',aggregated_pkl_file_suffix='freqly_to_mix.pkl', aggregated_pkl_mapping_file_suffix = 'freqly_to_mix_mapping.pkl', list_pkl_file_suffix = 'my_list.pkl', local_root_directory='../data/', user = 'napoleon', add_turn_over = False, skipping_size=200, recompute_all = True, db_path_suffix=None):
         super().__init__()
         self.freqly_return_pkl_filename_suffix=freqly_return_pkl_filename_suffix
-        self.aggregated_pkl_file_name=aggregated_pkl_file_name
-        self.aggregated_pkl_mapping_file_name=aggregated_pkl_mapping_file_name
+        self.aggregated_pkl_file_suffix=aggregated_pkl_file_suffix
+        self.aggregated_pkl_mapping_file_suffix=aggregated_pkl_mapping_file_suffix
         self.underlying = underlying
-        self.list_pkl_file_name = list_pkl_file_name
+        self.list_pkl_file_suffix = list_pkl_file_suffix
+        self.frequence=frequence
+        self.algo=algo
+
+        self.list_pkl_file_name = self.underlying + '_' + self.frequence + '_' + self.algo + self.list_pkl_file_suffix
+        self.aggregated_pkl_file_name = self.underlying + '_' + self.frequence + '_' + self.algo + self.aggregated_pkl_file_suffix
+        self.aggregated_pkl_mapping_file_name = self.underlying + '_' + self.frequence + '_' + self.algo + self.aggregated_pkl_mapping_file_suffix
+
         self.local_root_directory=local_root_directory
         self.user=user
         self.dropbox_backup = dropbox_backup
@@ -48,6 +56,25 @@ class AbstractRunner(ABC):
 
 
 class EnsemblingSignalRunner(AbstractRunner):
+
+    def backup_to_dropbox(self):
+
+        local_root_directory = Path(self.local_root_directory)
+        aggregated_csv_file_name = self.aggregated_pkl_file_name.replace('pkl', 'csv')
+        aggregated_csv_mapping_file_name = self.aggregated_pkl_mapping_file_name.replace('pkl', 'csv')
+
+        aggregated_df = pd.read_pickle(local_root_directory / self.aggregated_pkl_file_name)
+        aggregated_mapping_df = pd.read_pickle(local_root_directory / self.aggregated_pkl_mapping_file_name)
+
+        aggregated_csv_full_path = local_root_directory / aggregated_csv_file_name
+        aggregated_csv_mapping_full_path = local_root_directory / aggregated_csv_mapping_file_name
+
+        aggregated_df.to_csv(aggregated_csv_full_path)
+        aggregated_mapping_df.to_csv(aggregated_csv_mapping_full_path)
+
+        self.dbx.upload(fullname=aggregated_csv_full_path, folder='', subfolder='', name=aggregated_csv_file_name,overwrite=True)
+        self.dbx.upload(fullname=aggregated_csv_mapping_full_path, folder='', subfolder='',name=aggregated_csv_mapping_file_name, overwrite=True)
+
     def runTrial(self, saver, seed, trigger, transaction_costs, normalization):
         cumulated_signals = None
         print('Number of signals to aggregate '+str(len(self.signals_list)))
